@@ -1,18 +1,18 @@
 use core::fmt;
 use std::borrow::Cow;
-use std::collections::{BTreeMap, BinaryHeap, HashMap, HashSet};
+use std::collections::{BTreeMap, BinaryHeap, HashMap, HashSet, VecDeque};
 use std::ffi::{CStr, CString};
 use std::hash::{BuildHasher, Hash};
 use std::marker;
 
-use crate::de::{Decode, Decoder, MapDecoder, MapEntryDecoder, ReferenceVisitor, SequenceDecoder};
+use crate::de::{Decode, Decoder, PairDecoder, PairsDecoder, ReferenceVisitor, SequenceDecoder};
 use crate::en::{Encode, Encoder, PairEncoder, SequenceEncoder};
 use crate::error::Error;
 use crate::internal::size_hint;
 
 impl Encode for String {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -39,12 +39,17 @@ impl<'de> Decode<'de> for String {
             type Error = E;
 
             #[inline]
-            fn expected(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-                write!(f, "expected string")
+            fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, "a string")
             }
 
             #[inline]
-            fn visit(self, string: &str) -> Result<Self::Ok, Self::Error> {
+            fn visit_borrowed(self, string: &'de str) -> Result<Self::Ok, Self::Error> {
+                self.visit_any(string)
+            }
+
+            #[inline]
+            fn visit_any(self, string: &str) -> Result<Self::Ok, Self::Error> {
                 Ok(string.to_owned())
             }
         }
@@ -53,7 +58,7 @@ impl<'de> Decode<'de> for String {
 
 impl Encode for Box<str> {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -73,7 +78,7 @@ impl<'de> Decode<'de> for Box<str> {
 
 impl Encode for Cow<'_, str> {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -100,17 +105,17 @@ impl<'de> Decode<'de> for Cow<'de, str> {
             type Error = E;
 
             #[inline]
-            fn expected(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-                write!(f, "expected string")
+            fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, "a string")
             }
 
             #[inline]
-            fn visit_ref(self, string: &'de str) -> Result<Self::Ok, Self::Error> {
+            fn visit_borrowed(self, string: &'de str) -> Result<Self::Ok, Self::Error> {
                 Ok(Cow::Borrowed(string))
             }
 
             #[inline]
-            fn visit(self, string: &str) -> Result<Self::Ok, Self::Error> {
+            fn visit_any(self, string: &str) -> Result<Self::Ok, Self::Error> {
                 Ok(Cow::Owned(string.to_owned()))
             }
         }
@@ -130,17 +135,17 @@ macro_rules! sequence {
             $($extra: $extra_bound0 $(+ $extra_bound)*),*
         {
             #[inline]
-            fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+            fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
             where
                 E: Encoder,
             {
                 let mut seq = encoder.encode_sequence(self.len())?;
 
                 for value in self {
-                    value.encode(seq.encode_next()?)?;
+                    value.encode(seq.next()?)?;
                 }
 
-                seq.finish()
+                seq.end()
             }
         }
 
@@ -157,7 +162,7 @@ macro_rules! sequence {
                 let mut $access = decoder.decode_sequence()?;
                 let mut out = $factory;
 
-                while let Some(value) = $access.decode_next()? {
+                while let Some(value) = $access.next()? {
                     out.$insert(T::decode(value)?);
                 }
 
@@ -173,11 +178,17 @@ sequence!(
     seq,
     Vec::with_capacity(size_hint::cautious(seq.size_hint()))
 );
+sequence!(
+    VecDeque<T>,
+    push_back,
+    seq,
+    VecDeque::with_capacity(size_hint::cautious(seq.size_hint()))
+);
 sequence!(
     HashSet<T: Eq + Hash, S: BuildHasher + Default>,
     insert,
-    set,
-    HashSet::with_capacity_and_hasher(size_hint::cautious(set.size_hint()), S::default())
+    seq,
+    HashSet::with_capacity_and_hasher(size_hint::cautious(seq.size_hint()), S::default())
 );
 sequence!(
     BinaryHeap<T: Ord>,
@@ -199,20 +210,20 @@ macro_rules! map {
             $($extra: $extra_bound0 $(+ $extra_bound)*),*
         {
             #[inline]
-            fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+            fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
             where
                 E: Encoder,
             {
                 let mut map = encoder.encode_map(self.len())?;
 
                 for (k, v) in self {
-                    let entry = map.encode_first()?;
+                    let entry = map.first()?;
                     Encode::encode(k, entry)?;
-                    let value = map.encode_second()?;
+                    let value = map.second()?;
                     Encode::encode(v, value)?;
                 }
 
-                map.finish()
+                map.end()
             }
         }
 
@@ -230,9 +241,9 @@ macro_rules! map {
                 let mut $access = decoder.decode_map()?;
                 let mut out = $with_capacity;
 
-                while let Some(mut entry) = $access.decode_entry()? {
-                    let key = K::decode(entry.decode_key()?)?;
-                    let value = V::decode(entry.decode_value()?)?;
+                while let Some(mut entry) = $access.next()? {
+                    let key = K::decode(entry.first()?)?;
+                    let value = V::decode(entry.second()?)?;
                     out.insert(key, value);
                 }
 
@@ -255,7 +266,7 @@ map!(
 
 impl Encode for CStr {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -276,7 +287,7 @@ impl<'de> Decode<'de> for &'de CStr {
 
 impl Encode for CString {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
