@@ -15,12 +15,12 @@ use core::sync::atomic::{
 use core::{fmt, marker};
 
 use crate::de::{Decode, Decoder, PairDecoder, ReferenceVisitor};
-use crate::en::{Encode, Encoder, VariantEncoder};
+use crate::en::{Encode, Encoder, PairEncoder};
 use crate::error::Error;
 
 impl Encode for () {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -40,19 +40,20 @@ impl<'de> Decode<'de> for () {
 
 impl<T> Encode for marker::PhantomData<T> {
     #[inline]
-    fn encode<E>(&self, _: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
-        Ok(())
+        encoder.encode_unit()
     }
 }
 
 impl<'de, T> Decode<'de> for marker::PhantomData<T> {
-    fn decode<D>(_: D) -> Result<Self, D::Error>
+    fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
     {
+        let () = decoder.decode_unit()?;
         Ok(marker::PhantomData)
     }
 }
@@ -86,7 +87,7 @@ macro_rules! non_zero {
     ($ty:ty) => {
         impl Encode for $ty {
             #[inline]
-            fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+            fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
             where
                 E: Encoder,
             {
@@ -103,7 +104,7 @@ macro_rules! non_zero {
 
                 match Self::new(value) {
                     Some(value) => Ok(value),
-                    None => Err(D::Error::collect_from_display(NonZeroUnsupportedValue {
+                    None => Err(D::Error::message(NonZeroUnsupportedValue {
                         type_name: stringify!($ty),
                         value,
                     })),
@@ -146,7 +147,7 @@ where
 
 impl<const N: usize> Encode for [u8; N] {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -168,7 +169,7 @@ macro_rules! impl_number {
     ($ty:ty, $read:ident, $write:ident) => {
         impl Encode for $ty {
             #[inline]
-            fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+            fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
             where
                 E: Encoder,
             {
@@ -190,7 +191,7 @@ macro_rules! impl_number {
 
 impl Encode for bool {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -210,7 +211,7 @@ impl<'de> Decode<'de> for bool {
 
 impl Encode for char {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -245,7 +246,7 @@ impl_number!(f64, decode_f64, encode_f64);
 
 impl Encode for str {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -272,12 +273,12 @@ impl<'de> Decode<'de> for &'de str {
             type Error = E;
 
             #[inline]
-            fn expected(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-                write!(f, "expected exact string reference")
+            fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, "string borrowed from source")
             }
 
             #[inline]
-            fn visit_ref(self, string: &'de str) -> Result<Self::Ok, Self::Error> {
+            fn visit_borrowed(self, string: &'de str) -> Result<Self::Ok, Self::Error> {
                 Ok(string)
             }
         }
@@ -285,7 +286,7 @@ impl<'de> Decode<'de> for &'de str {
 }
 
 impl Encode for [u8] {
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -312,12 +313,12 @@ impl<'de> Decode<'de> for &'de [u8] {
             type Error = E;
 
             #[inline]
-            fn expected(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-                write!(f, "expected exact bytes reference")
+            fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, "bytes borrowed from source")
             }
 
             #[inline]
-            fn visit_ref(self, bytes: &'de [u8]) -> Result<Self::Ok, Self::Error> {
+            fn visit_borrowed(self, bytes: &'de [u8]) -> Result<Self::Ok, Self::Error> {
                 Ok(bytes)
             }
         }
@@ -329,7 +330,7 @@ where
     T: Encode,
 {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -366,7 +367,7 @@ where
     U: Encode,
 {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
@@ -374,14 +375,14 @@ where
 
         match self {
             Ok(ok) => {
-                Encode::encode(&0usize, variant.encode_variant_tag()?)?;
-                Encode::encode(ok, variant.encode_variant_value()?)?;
-                Ok(())
+                Encode::encode(&0usize, variant.first()?)?;
+                Encode::encode(ok, variant.second()?)?;
+                variant.end()
             }
             Err(err) => {
-                Encode::encode(&1usize, variant.encode_variant_tag()?)?;
-                Encode::encode(err, variant.encode_variant_value()?)?;
-                Ok(())
+                Encode::encode(&1usize, variant.first()?)?;
+                Encode::encode(err, variant.second()?)?;
+                variant.end()
             }
         }
     }
@@ -399,10 +400,10 @@ where
     {
         let mut variant = decoder.decode_variant()?;
 
-        match variant.decode_first().and_then(usize::decode)? {
-            0 => variant.decode_second().and_then(T::decode).map(Ok),
-            1 => variant.decode_second().and_then(U::decode).map(Err),
-            tag => Err(D::Error::unsupported_variant("Result", tag)),
+        match variant.first().and_then(usize::decode)? {
+            0 => variant.second().and_then(T::decode).map(Ok),
+            1 => variant.second().and_then(U::decode).map(Err),
+            tag => Err(D::Error::invalid_variant_tag("Result", tag)),
         }
     }
 }
@@ -412,7 +413,7 @@ where
     T: Encode,
 {
     #[inline]
-    fn encode<E>(&self, encoder: E) -> Result<(), E::Error>
+    fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
